@@ -19,14 +19,23 @@
 
 from __future__ import absolute_import, print_function
 
+from collections import OrderedDict
+from functools import wraps
 import re
+import pickle
+import base64
+import zlib
+from functools import reduce
 try:
-    from urllib.parse import urlparse, urljoin
+    from requests.packages import urllib3
 except ImportError:
-    from urlparse import urlparse, urljoin
+    import urllib3
 import os
 import sys
 from copy import deepcopy
+import inspect
+from datetime import datetime, timedelta
+from dateutil import parser
 
 try:
     import requests
@@ -35,15 +44,18 @@ try:
 except ImportError:
     raise ImportError('Please install python-requests >= 2.0')
 
+from weboob.exceptions import BrowserHTTPSDowngrade, ModuleInstallError
+
 from weboob.tools.log import getLogger
-from weboob.tools.ordereddict import OrderedDict
+from weboob.tools.compat import basestring, unicode, urlparse, urljoin
+from weboob.tools.json import json
 
 from .cookies import WeboobCookieJar
 from .exceptions import HTTPNotFound, ClientError, ServerError
 from .sessions import FuturesSession
 from .profiles import Firefox
 from .pages import NextPage
-from .url import URL
+from .url import URL, normalize_url
 
 
 class Browser(object):
@@ -82,17 +94,40 @@ class Browser(object):
     Maximum of threads for asynchronous requests.
     """
 
-    def __init__(self, logger=None, proxy=None, responses_dirname=None):
+    ALLOW_REFERRER = True
+    """
+    Controls the behavior of get_referrer.
+    """
+
+    @classmethod
+    def asset(cls, localfile):
+        """
+        Absolute file path for a module local file.
+        """
+        if os.path.isabs(localfile):
+            return localfile
+        return os.path.join(os.path.dirname(inspect.getfile(cls)), localfile)
+
+    def __init__(self, logger=None, proxy=None, responses_dirname=None, weboob=None):
         self.logger = getLogger('browser', logger)
+        self.responses_dirname = responses_dirname
+        self.responses_count = 1
+
+        if isinstance(self.VERIFY, basestring):
+            self.VERIFY = self.asset(self.VERIFY)
+
         self.PROXIES = proxy
         self._setup_session(self.PROFILE)
         self.url = None
         self.response = None
 
-        self.responses_dirname = responses_dirname
-        self.responses_count = 1
+    def deinit(self):
+        self.session.close()
 
-    def _save(self, response, warning=False, **kwargs):
+    def set_normalized_url(self, response, **kwargs):
+        response.url = normalize_url(response.url)
+
+    def save_response(self, response, warning=False, **kwargs):
         if self.responses_dirname is None:
             import tempfile
             self.responses_dirname = tempfile.mkdtemp(prefix='weboob_session_')
@@ -117,13 +152,11 @@ class Browser(object):
             (self.responses_count, response.status_code, '-' if path else '', path, ext)
 
         response_filepath = os.path.join(self.responses_dirname, filename)
-        with open(response_filepath, 'w') as f:
-            f.write(response.content)
 
         request = response.request
         with open(response_filepath + '-request.txt', 'w') as f:
             f.write('%s %s\n\n\n' % (request.method, request.url))
-            for key, value in request.headers.iteritems():
+            for key, value in request.headers.items():
                 f.write('%s: %s\n' % (key, value))
             if request.body is not None:  # separate '' from None
                 f.write('\n\n\n%s' % request.body)
@@ -131,9 +164,12 @@ class Browser(object):
             if hasattr(response.elapsed, 'total_seconds'):
                 f.write('Time: %3.3fs\n' % response.elapsed.total_seconds())
             f.write('%s %s\n\n\n' % (response.status_code, response.reason))
-            for key, value in response.headers.iteritems():
+            for key, value in response.headers.items():
                 f.write('%s: %s\n' % (key, value))
 
+        with open(response_filepath, 'wb') as f:
+            f.write(response.content)
+
         match_filepath = os.path.join(self.responses_dirname, 'url_response_match.txt')
         with open(match_filepath, 'a') as f:
             f.write('# %d %s %s\n' % (response.status_code, response.reason, response.headers.get('Content-Type', '')))
@@ -146,21 +182,34 @@ class Browser(object):
         else:
             self.logger.info(msg)
 
+    def _create_session(self):
+        return FuturesSession(max_workers=self.MAX_WORKERS, max_retries=self.MAX_RETRIES)
+
     def _setup_session(self, profile):
         """
         Set up a python-requests session for our usage.
         """
-        session = FuturesSession(max_workers=self.MAX_WORKERS)
+        session = self._create_session()
 
         session.proxies = self.PROXIES
 
-        session.verify = self.VERIFY and not self.logger.settings['ssl_insecure']
+        session.verify = not self.logger.settings['ssl_insecure'] and self.VERIFY
+        if not session.verify:
+            try:
+                urllib3.disable_warnings()
+            except AttributeError:
+                # urllib3 is too old, warnings won't be disable
+                pass
 
         # defines a max_retries. It's mandatory in case a server is not
         # handling keep alive correctly, like the proxy burp
-        a = requests.adapters.HTTPAdapter(max_retries=self.MAX_RETRIES)
-        session.mount('http://', a)
-        session.mount('https://', a)
+        adapter_kwargs = dict(max_retries=self.MAX_RETRIES)
+        # set connection pool size equal to MAX_WORKERS if needed
+        if self.MAX_WORKERS > requests.adapters.DEFAULT_POOLSIZE:
+            adapter_kwargs.update(pool_connections=self.MAX_WORKERS,
+                                  pool_maxsize=self.MAX_WORKERS)
+        session.mount('https://', requests.adapters.HTTPAdapter(**adapter_kwargs))
+        session.mount('http://', requests.adapters.HTTPAdapter(**adapter_kwargs))
 
         if self.TIMEOUT:
             session.timeout = self.TIMEOUT
@@ -169,13 +218,17 @@ class Browser(object):
 
         profile.setup_session(session)
 
-        if self.logger.settings['save_responses']:
-            session.hooks['response'].append(self._save)
+        session.hooks['response'].append(self.set_normalized_url)
+        if self.responses_dirname is not None:
+            session.hooks['response'].append(self.save_response)
 
         self.session = session
 
         session.cookies = WeboobCookieJar()
 
+    def set_profile(self, profile):
+        profile.setup_session(self.session)
+
     def location(self, url, **kwargs):
         """
         Like :meth:`open` but also changes the current URL and response.
@@ -183,7 +236,7 @@ class Browser(object):
 
         Other than that, has the exact same behavior of open().
         """
-        assert not kwargs.get('async'), "Please use open() instead of location() to make asynchronous requests."
+        assert not kwargs.get('is_async'), "Please use open() instead of location() to make asynchronous requests."
         response = self.open(url, **kwargs)
         self.response = response
         self.url = self.response.url
@@ -197,7 +250,7 @@ class Browser(object):
                    cert=None,
                    proxies=None,
                    data_encoding=None,
-                   async=False,
+                   is_async=False,
                    callback=lambda response: response,
                    **kwargs):
         """
@@ -218,14 +271,14 @@ class Browser(object):
         Call this instead of location() if you do not want to "visit" the URL
         (for instance, you are downloading a file).
 
-        When `async` is True, open() returns a Future objet (see
+        When `is_async` is True, open() returns a Future object (see
         concurrent.futures for more details), which can be evaluated with its
         result() method. If any exception is raised while processing request,
-        it is catched and re-raised when calling result().
+        it is caught and re-raised when calling result().
 
         For example:
 
-        >>> Browser().open('http://google.com', async=True).result().text # doctest: +SKIP
+        >>> Browser().open('http://google.com', is_async=True).result().text # doctest: +SKIP
 
         :param url: URL
         :type url: str
@@ -236,8 +289,8 @@ class Browser(object):
         :param referrer: Force referrer. False to disable sending it, None for guessing
         :type referrer: str or False or None
 
-        :param async: Process request in a non-blocking way
-        :type async: bool
+        :param is_async: Process request in a non-blocking way
+        :type is_async: bool
 
         :param callback: Callback to be called when request has finished,
                          with response as its first and only argument
@@ -245,6 +298,17 @@ class Browser(object):
 
         :rtype: :class:`requests.Response`
         """
+        if 'async' in kwargs:
+            import warnings
+            warnings.warn('Please use is_async instead of async.', DeprecationWarning)
+            is_async = kwargs['async']
+            del kwargs['async']
+
+        if isinstance(url, basestring):
+            url = normalize_url(url)
+        elif isinstance(url, requests.Request):
+            url.url = normalize_url(url.url)
+
         req = self.build_request(url, referrer, data_encoding=data_encoding, **kwargs)
         preq = self.prepare_request(req)
 
@@ -258,13 +322,13 @@ class Browser(object):
             proxies = self.PROXIES
 
         if verify is None:
-            verify = self.VERIFY and not self.logger.settings['ssl_insecure']
+            verify = not self.logger.settings['ssl_insecure'] and self.VERIFY
 
         if timeout is None:
             timeout = self.TIMEOUT
 
         # We define an inner_callback here in order to execute the same code
-        # regardless of async param.
+        # regardless of is_async param.
         def inner_callback(future, response):
             if allow_redirects:
                 response = self.handle_refresh(response)
@@ -280,19 +344,19 @@ class Browser(object):
                                      verify=verify,
                                      cert=cert,
                                      proxies=proxies,
-                                     background_callback=async and inner_callback)
-        if not async:
-            inner_callback(self, response)
-
+                                     callback=inner_callback,
+                                     is_async=is_async)
         return response
 
     def async_open(self, url, **kwargs):
         """
-        Shortcut to open(url, async=True).
+        Shortcut to open(url, is_async=True).
         """
         if 'async' in kwargs:
             del kwargs['async']
-        return self.open(url, async=True, **kwargs)
+        if 'is_async' in kwargs:
+            del kwargs['is_async']
+        return self.open(url, is_async=True, **kwargs)
 
     def raise_for_status(self, response):
         """
@@ -338,8 +402,8 @@ class Browser(object):
         if isinstance(req.data, unicode) and data_encoding:
             req.data = req.data.encode(data_encoding)
         if isinstance(req.data, dict) and data_encoding:
-            req.data = dict([(k, v.encode(data_encoding) if isinstance(v, unicode) else v)
-                             for k, v in req.data.iteritems()])
+            req.data = OrderedDict([(k, v.encode(data_encoding) if isinstance(v, unicode) else v)
+                                    for k, v in req.data.items()])
 
         if referrer is None:
             referrer = self.get_referrer(self.url, url)
@@ -393,6 +457,11 @@ class Browser(object):
 
         Reference: https://en.wikipedia.org/wiki/HTTP_referer
 
+        The behavior can be controlled through the ALLOW_REFERRER attribute.
+        True always allows the referers
+        to be sent, False never, and None only if it is within
+        the same domain.
+
         :param oldurl: Current absolute URL
         :type oldurl: str or None
 
@@ -401,23 +470,38 @@ class Browser(object):
 
         :rtype: str or None
         """
+        if self.ALLOW_REFERRER is False:
+            return
         if oldurl is None:
-            return None
+            return
         old = urlparse(oldurl)
         new = urlparse(newurl)
         # Do not leak secure URLs to insecure URLs
         if old.scheme == 'https' and new.scheme != 'https':
-            return None
+            return
         # Reloading the page. Usually no referrer.
         if oldurl == newurl:
-            return None
-        # TODO maybe implement some *optional* privacy features:
-        # * do not leak referrer to other domains (often breaks websites)
-        # * send a fake referrer (root of the current domain)
-        # * never send the referrer
-        # Inspired by the RefControl Firefox addon.
+            return
+        # Domain-based privacy
+        if self.ALLOW_REFERRER is None and old.netloc != new.netloc:
+            return
         return oldurl
 
+    def export_session(self):
+        def make_cookie(c):
+            d = {
+                k: getattr(c, k) for k in ['name', 'value', 'domain', 'path', 'secure']
+            }
+            #d['session'] = c.discard
+            d['httpOnly'] = 'httponly' in [k.lower() for k in c._rest.keys()]
+            d['expirationDate'] = getattr(c, 'expires', None)
+            return d
+
+        return {
+            'url': self.url,
+            'cookies': [make_cookie(c) for c in self.session.cookies],
+        }
+
 
 class UrlNotAllowed(Exception):
     """
@@ -449,6 +533,11 @@ class DomainBrowser(Browser):
     More complex behavior is possible by overloading url_allowed()
     """
 
+    def __init__(self, baseurl=None, *args, **kwargs):
+        super(DomainBrowser, self).__init__(*args, **kwargs)
+        if baseurl is not None:
+            self.BASEURL = baseurl
+
     def url_allowed(self, url):
         """
         Checks if we are allowed to visit an URL.
@@ -469,15 +558,18 @@ class DomainBrowser(Browser):
 
     def absurl(self, uri, base=None):
         """
-        Get the absolute URL, relative to the base URL.
-        If BASEURL is None, it will try to use the current URL.
+        Get the absolute URL, relative to a base URL.
+        If base is None, it will try to use the current URL.
+        If there is no current URL, it will try to use BASEURL.
+
         If base is False, it will always try to use the current URL.
+        If base is True, it will always try to use BASEURL.
 
         :param uri: URI to make absolute. It can be already absolute.
         :type uri: str
 
         :param base: Base absolute URL.
-        :type base: str or None or False
+        :type base: str or None or False or True
 
         :rtype: str
         """
@@ -489,7 +581,7 @@ class DomainBrowser(Browser):
 
     def open(self, req, *args, **kwargs):
         """
-        Like :meth:`Browser.open` but hanldes urls without domains, using
+        Like :meth:`Browser.open` but handles urls without domains, using
         the :attr:`BASEURL` attribute.
         """
         uri = req.url if isinstance(req, requests.Request) else req
@@ -511,23 +603,6 @@ class DomainBrowser(Browser):
         return self.location(self.BASEURL or self.absurl('/'))
 
 
-class _PagesBrowserMeta(type):
-    """
-    Private meta-class used to keep order of URLs instances of PagesBrowser.
-    """
-    def __new__(mcs, name, bases, attrs):
-        urls = [(url_name, attrs.pop(url_name)) for url_name, obj in attrs.items() if isinstance(obj, URL)]
-        urls.sort(key=lambda x: x[1]._creation_counter)
-
-        new_class = super(_PagesBrowserMeta, mcs).__new__(mcs, name, bases, attrs)
-        if new_class._urls is None:
-            new_class._urls = OrderedDict()
-        else:
-            new_class._urls = deepcopy(new_class._urls)
-        new_class._urls.update(urls)
-        return new_class
-
-
 class PagesBrowser(DomainBrowser):
     r"""
     A browser which works pages and keep state of navigation.
@@ -539,11 +614,11 @@ class PagesBrowser(DomainBrowser):
 
     Example:
 
-    >>> from .pages import Page
-    >>> class HomePage(Page):
+    >>> from .pages import HTMLPage
+    >>> class HomePage(HTMLPage):
     ...     pass
     ...
-    >>> class ListPage(Page):
+    >>> class ListPage(HTMLPage):
     ...     pass
     ...
     >>> class MyBrowser(PagesBrowser):
@@ -555,23 +630,26 @@ class PagesBrowser(DomainBrowser):
     You can then use URL instances to go on pages.
     """
 
-
     _urls = None
-    __metaclass__ = _PagesBrowserMeta
-
-    def __getattr__(self, name):
-        if self._urls is not None and name in self._urls:
-            return self._urls[name]
-        else:
-            raise AttributeError("'%s' object has no attribute '%s'" % (
-                self.__class__.__name__, name))
 
     def __init__(self, *args, **kwargs):
+        self.highlight_el = kwargs.pop('highlight_el', False)
         super(PagesBrowser, self).__init__(*args, **kwargs)
 
         self.page = None
-        self._urls = deepcopy(self._urls)
-        for url in self._urls.itervalues():
+
+        # exclude properties because they can access other fields not yet defined
+        def is_property(attr):
+            v = getattr(type(self), attr, None)
+            return hasattr(v, '__get__') or hasattr(v, '__set__')
+
+        attrs = [(attr, getattr(self, attr)) for attr in dir(self) if not is_property(attr)]
+        attrs = [v for v in attrs if isinstance(v[1], URL)]
+        attrs.sort(key=lambda v: v[1]._creation_counter)
+        self._urls = OrderedDict(deepcopy(attrs))
+        for k, v in self._urls.items():
+            setattr(self, k, v)
+        for url in self._urls.values():
             url.browser = self
 
     def open(self, *args, **kwargs):
@@ -585,12 +663,12 @@ class PagesBrowser(DomainBrowser):
         callback = kwargs.pop('callback', lambda response: response)
 
         # Have to define a callback to seamlessly process synchronous and
-        # asynchronous requests, see :meth:`Browser.open` and its `async`
+        # asynchronous requests, see :meth:`Browser.open` and its `is_async`
         # and `callback` params.
         def internal_callback(response):
             # Try to handle the response page with an URL instance.
             response.page = None
-            for url in self._urls.itervalues():
+            for url in self._urls.values():
                 page = url.handle(response)
                 if page is not None:
                     self.logger.debug('Handle %s with %s' % (response.url, page.__class__.__name__))
@@ -598,6 +676,16 @@ class PagesBrowser(DomainBrowser):
                     break
 
             if response.page is None:
+                regexp = r'^(?P<proto>\w+)://.*'
+
+                proto_response = re.match(regexp, response.url)
+                if proto_response:
+                    proto_response = proto_response.group('proto')
+                    proto_base = re.match(regexp, self.BASEURL).group('proto')
+
+                    if proto_base == 'https' and proto_response != 'https':
+                        raise BrowserHTTPSDowngrade()
+
                 self.logger.debug('Unable to handle %s' % response.url)
 
             return callback(response)
@@ -646,7 +734,7 @@ class PagesBrowser(DomainBrowser):
         ...             raise NextPage(next.attrib['href'])
         ...
         >>> class Browser(PagesBrowser):
-        ...     BASEURL = 'http://people.symlink.me'
+        ...     BASEURL = 'https://people.symlink.me'
         ...     list = URL('/~rom1/projects/weboob/list-(?P<pagenum>\d+).html', Page)
         ...
         >>> b = Browser()
@@ -669,9 +757,14 @@ def need_login(func):
     """
     Decorator used to require to be logged to access to this function.
     """
+
+    @wraps(func)
     def inner(browser, *args, **kwargs):
-        if browser.page is None or not browser.page.logged:
+        if (not hasattr(browser, 'logged') or (hasattr(browser, 'logged') and not browser.logged)) and \
+                (not hasattr(browser, 'page') or browser.page is None or not browser.page.logged):
             browser.do_login()
+            if browser.logger.settings.get('export_session'):
+                browser.logger.debug('logged in with session: %s', json.dumps(browser.export_session()))
         return func(browser, *args, **kwargs)
 
     return inner
@@ -694,3 +787,146 @@ class LoginBrowser(PagesBrowser):
         It is call when a login is needed.
         """
         raise NotImplementedError()
+
+    def do_logout(self):
+        """
+        Logout from website.
+
+        By default, simply clears the cookies.
+        """
+        self.session.cookies.clear()
+
+
+class StatesMixin(object):
+    """
+    Mixin to store states of browser.
+    """
+
+    __states__ = []
+    """
+    Saved state variables.
+    """
+
+    STATE_DURATION = None
+    """
+    In minutes, used to set an expiration datetime object of the state.
+    """
+
+    def locate_browser(self, state):
+        try:
+            self.location(state['url'])
+        except (requests.exceptions.HTTPError, requests.exceptions.TooManyRedirects):
+            pass
+
+    def load_state(self, state):
+        if 'expire' in state and parser.parse(state['expire']) < datetime.now():
+            return self.logger.info('State expired, not reloading it from storage')
+        if 'cookies' in state:
+            try:
+                self.session.cookies = pickle.loads(zlib.decompress(base64.b64decode(state['cookies'])))
+            except (TypeError, zlib.error, EOFError, ValueError):
+                self.logger.error('Unable to reload cookies from storage')
+            else:
+                self.logger.info('Reloaded cookies from storage')
+        for attrname in self.__states__:
+            if attrname in state:
+                setattr(self, attrname, state[attrname])
+
+        if 'url' in state:
+            self.locate_browser(state)
+
+    def dump_state(self):
+        state = {}
+        if hasattr(self, 'page') and self.page:
+            state['url'] = self.page.url
+        state['cookies'] = base64.b64encode(zlib.compress(pickle.dumps(self.session.cookies, -1)))
+        for attrname in self.__states__:
+            try:
+                state[attrname] = getattr(self, attrname)
+            except AttributeError:
+                pass
+        if self.STATE_DURATION is not None:
+            state['expire'] = unicode((datetime.now() + timedelta(minutes=self.STATE_DURATION)).replace(microsecond=0))
+        self.logger.info('Stored cookies into storage')
+        return state
+
+
+class APIBrowser(DomainBrowser):
+    """
+    A browser for API websites.
+    """
+
+    def build_request(self, *args, **kwargs):
+        if 'data' in kwargs:
+            kwargs['data'] = json.dumps(kwargs['data'])
+        if 'headers' not in kwargs:
+            kwargs['headers'] = {}
+        kwargs['headers']['Content-Type'] = 'application/json'
+
+        return super(APIBrowser, self).build_request(*args, **kwargs)
+
+    def open(self, *args, **kwargs):
+        """
+        Do a JSON request.
+
+        The "Content-Type" header is always set to "application/json".
+
+        :param data: if specified, format as JSON and send as request body
+        :type data: :class:`dict`
+        :param headers: if specified, add these headers to the request
+        :type headers: :class:`dict`
+        """
+        return super(APIBrowser, self).open(*args, **kwargs)
+
+    def request(self, *args, **kwargs):
+        """
+        Do a JSON request and parse the response.
+
+        :returns: a dict containing the parsed JSON server response
+        :rtype: :class:`dict`
+        """
+        return self.open(*args, **kwargs).json()
+
+
+class AbstractBrowserMissingParentError(Exception):
+    pass
+
+
+class AbstractBrowser(Browser):
+    """ AbstractBrowser allow inheritance of a browser defined in another module.
+
+    Websites can share many pages and code base. This class allow to load a browser
+    provided by another module and to build our own browser on top of it (like standard
+    python inheritance. Weboob will install and download the PARENT module for you.
+
+    PARENT is a mandatory attribute, it's the name of the module providing the parent Browser
+
+    PARENT_ATTR is an optionnal attribute used when the parent module does not have only one
+    browser defined as BROWSER class attribute: you can customized the path of the object to load.
+
+    Note that you must pass a valid weboob instance as first argument of the constructor.
+    """
+    PARENT = None
+    PARENT_ATTR = None
+
+    def __new__(cls, *args, **kwargs):
+        weboob = kwargs['weboob']
+
+        if cls.PARENT is None:
+            raise AbstractBrowserMissingParentError("PARENT is not defined for browser %s" % cls)
+
+        try:
+            module = weboob.load_or_install_module(cls.PARENT)
+        except ModuleInstallError as err:
+            raise ModuleInstallError('This module depends on %s module but %s\'s installation failed with: %s' % (cls.PARENT, cls.PARENT, err))
+
+        if cls.PARENT_ATTR is None:
+            parent = module.klass.BROWSER
+        else:
+            parent = reduce(getattr, cls.PARENT_ATTR.split('.'), module)
+
+        if parent is None:
+            raise AbstractBrowserMissingParentError("Failed to load parent class")
+
+        cls.__bases__ = (parent,)
+        return object.__new__(cls)
