@@ -21,35 +21,29 @@ from __future__ import absolute_import
 
 import datetime
 import re
+import unicodedata
 from decimal import Decimal, InvalidOperation
 from itertools import islice
+from collections import Iterator
 
 from dateutil.parser import parse as parse_date
 
 from weboob.capabilities.base import empty
-from weboob.tools.compat import basestring
+from weboob.capabilities.base import Currency as BaseCurrency
+from weboob.tools.compat import basestring, unicode, long
 from weboob.exceptions import ParseError
 from weboob.browser.url import URL
-from weboob.tools.log import getLogger, DEBUG_FILTERS
+from weboob.tools.compat import parse_qs, urlparse
 
-
-class NoDefault(object):
-    def __repr__(self):
-        return 'NO_DEFAULT'
-
-_NO_DEFAULT = NoDefault()
+from .base import _NO_DEFAULT, FilterError, _Filter, Filter, debug
 
 
 __all__ = ['FilterError', 'ColumnNotFound', 'RegexpError', 'ItemNotFound',
            'Filter', 'Base', 'Env', 'TableCell', 'RawText',
-           'CleanText', 'Lower', 'CleanDecimal', 'Field', 'Regexp', 'Map',
-           'DateTime', 'Date', 'Time', 'DateGuesser', 'Duration',
-           'MultiFilter', 'CombineDate', 'Format', 'Join', 'Type',
-           'BrowserURL', 'Async', 'AsyncLoad']
-
-
-class FilterError(ParseError):
-    pass
+           'CleanText', 'Lower', 'Upper', 'Capitalize', 'CleanDecimal',
+           'Field', 'Regexp', 'Map', 'DateTime', 'Date', 'Time', 'DateGuesser',
+           'Duration', 'MultiFilter', 'CombineDate', 'Format', 'Join', 'Type',
+           'Eval', 'BrowserURL', 'Async', 'AsyncLoad']
 
 
 class ColumnNotFound(FilterError):
@@ -64,137 +58,13 @@ class ItemNotFound(FilterError):
     pass
 
 
-class _Filter(object):
-    _creation_counter = 0
-
-    def __init__(self, default=_NO_DEFAULT):
-        self._key = None
-        self._obj = None
-        self.default = default
-        self._creation_counter = _Filter._creation_counter
-        _Filter._creation_counter += 1
-
-    def __or__(self, o):
-        self.default = o
-        return self
-
-    def __and__(self, o):
-        if isinstance(o, type) and issubclass(o, _Filter):
-            o = o()
-        o.selector = self
-        return o
-
-    def default_or_raise(self, exception):
-        if self.default is not _NO_DEFAULT:
-            return self.default
-        else:
-            raise exception
-
-    def __str__(self):
-        return self.__class__.__name__
-
-
-def debug(*args):
-    """
-    A decorator function to provide some debug information
-    in Filters.
-    It prints by default the name of the Filter and the input value.
-    """
-    def wraper(function):
-        def print_debug(self, value):
-            logger = getLogger('b2filters')
-            result = ''
-            outputvalue = value
-            if isinstance(value, list):
-                from lxml import etree
-                outputvalue = ''
-                first = True
-                for element in value:
-                    if first:
-                        first = False
-                    else:
-                        outputvalue += ', '
-                    if isinstance(element, etree.ElementBase):
-                        outputvalue += "%s" % etree.tostring(element, encoding=unicode)
-                    else:
-                        outputvalue += "%r" % element
-            if self._obj is not None:
-                result += "%s" % self._obj._random_id
-            if self._key is not None:
-                result += ".%s" % self._key
-            name = str(self)
-            result += " %s(%r" % (name, outputvalue)
-            for arg in self.__dict__:
-                if arg.startswith('_') or arg == u"selector":
-                    continue
-                if arg == u'default' and getattr(self, arg) == _NO_DEFAULT:
-                    continue
-                result += ", %s=%r" % (arg, getattr(self, arg))
-            result += u')'
-            logger.log(DEBUG_FILTERS, result)
-            res = function(self, value)
-            return res
-        return print_debug
-    return wraper
-
-
-class Filter(_Filter):
-    """
-    Class used to filter on a HTML element given as call parameter to return
-    matching elements.
-
-    Filters can be chained, so the parameter supplied to constructor can be
-    either a xpath selector string, or an other filter called before.
-
-    >>> from lxml.html import etree
-    >>> f = CleanDecimal(CleanText('//p'), replace_dots=True)
-    >>> f(etree.fromstring('<html><body><p>blah: <span>229,90</span></p></body></html>'))
-    Decimal('229.90')
-    """
-
-    def __init__(self, selector=None, default=_NO_DEFAULT):
-        super(Filter, self).__init__(default=default)
-        self.selector = selector
-
-    @classmethod
-    def select(cls, selector, item, obj=None, key=None):
-        if isinstance(selector, basestring):
-            return item.xpath(selector)
-        elif isinstance(selector, _Filter):
-            selector._key = key
-            selector._obj = obj
-            return selector(item)
-        elif callable(selector):
-            return selector(item)
-        else:
-            return selector
-
-    def __call__(self, item):
-        return self.filter(self.select(self.selector, item, key=self._key, obj=self._obj))
-
-    @debug()
-    def filter(self, value):
-        """
-        This method have to be overrided by children classes.
-        """
-        raise NotImplementedError()
-
-
-class _Selector(Filter):
-    def filter(self, elements):
-        if elements is not None:
-            return elements
-        else:
-            return self.default_or_raise(ParseError('Element %r not found' % self.selector))
-
-
 class AsyncLoad(Filter):
     def __call__(self, item):
-        link = self.select(self.selector, item, key=self._key, obj=self._obj)
-        return item.page.browser.async_open(link)
+        link = self.select(self.selector, item)
+        return item.page.browser.async_open(link) if link else None
 
 
-class Async(_Filter):
+class Async(Filter):
     def __init__(self, name, selector=None):
         super(Async, self).__init__()
         self.selector = selector
@@ -207,9 +77,18 @@ class Async(_Filter):
         return self
 
     def __call__(self, item):
+        if item.loaders[self.name] is None:
+            return None
+
+        return self.select(self.selector, self.loaded_page(item).doc)
+
+    def filter(self, *args):
+        raise AttributeError()
+
+    def loaded_page(self, item):
         result = item.loaders[self.name].result()
         assert result.page is not None, 'The loaded url %s hasn\'t been matched by an URL object' % result.url
-        return self.selector(result.page.doc)
+        return result.page
 
 
 class Base(Filter):
@@ -220,14 +99,40 @@ class Base(Filter):
     """
 
     def __call__(self, item):
-        base = self.select(self.base, item, obj=self._obj, key=self._key)
-        return self.selector(base)
+        base = self.select(self.base, item)
+        return self.select(self.selector, base)
 
     def __init__(self, base, selector=None, default=_NO_DEFAULT):
         super(Base, self).__init__(selector, default)
         self.base = base
 
 
+class Decode(Filter):
+    """
+    Filter that aims to decode urlencoded strings
+
+    >>> Decode(Env('_id'))  # doctest: +ELLIPSIS
+    <weboob.browser.filters.standard.Decode object at 0x...>
+    >>> from .html import Link
+    >>> Decode(Link('./a'))  # doctest: +ELLIPSIS
+    <weboob.browser.filters.standard.Decode object at 0x...>
+
+    """
+    def __call__(self, item):
+        self.encoding = item.page.ENCODING if item.page.ENCODING else 'utf-8'
+        return self.filter(self.select(self.selector, item))
+
+    @debug()
+    def filter(self, txt):
+        from weboob.tools.compat import unquote
+        try:
+            txt = unquote(txt.encode('ascii')).decode(self.encoding)
+        except (UnicodeDecodeError, UnicodeEncodeError):
+            pass
+
+        return txt
+
+
 class Env(_Filter):
     """
     Filter to get environment value of the item.
@@ -268,28 +173,48 @@ class TableCell(_Filter):
     """
 
     def __init__(self, *names, **kwargs):
+        support_th = kwargs.pop('support_th', False)
         super(TableCell, self).__init__(**kwargs)
         self.names = names
 
+        if support_th:
+            self.td = '(./th | ./td)[%s]'
+        else:
+            self.td = './td[%s]'
+
     def __call__(self, item):
         for name in self.names:
             idx = item.parent.get_colnum(name)
             if idx is not None:
-                return item.xpath('./td[%s]' % (idx + 1))
+                ret = item.xpath(self.td % (idx + 1))
+                for el in ret:
+                    self.highlight_el(el, item)
+                return ret
 
         return self.default_or_raise(ColumnNotFound('Unable to find column %s' % ' or '.join(self.names)))
 
 
 class RawText(Filter):
+    def __init__(self, selector=None, children=False, default=_NO_DEFAULT):
+        super(RawText, self).__init__(selector, default=default)
+        self.children = children
+
     @debug()
     def filter(self, el):
         if isinstance(el, (tuple, list)):
             return u' '.join([self.filter(e) for e in el])
 
-        if el.text is None:
-            return self.default
+        if self.children:
+            text = el.text_content()
         else:
-            return unicode(el.text)
+            text = el.text
+
+        if text is None:
+            result = self.default
+        else:
+            result = unicode(text)
+
+        return result
 
 
 class CleanText(Filter):
@@ -299,6 +224,10 @@ class CleanText(Filter):
     It first replaces all tabs and multiple spaces
     (including newlines if ``newlines`` is True)
     to one space and strips the result string.
+
+    The result is coerced into unicode, and optionally normalized
+    according to the ``normalize`` argument.
+
     Then it replaces all symbols given in the ``symbols`` argument.
 
     >>> CleanText().filter('coucou ')
@@ -311,38 +240,45 @@ class CleanText(Filter):
     u'coucou\\ncoucou'
     """
 
-    def __init__(self, selector=None, symbols='', replace=[], childs=True, newlines=True, **kwargs):
+    def __init__(self, selector=None, symbols='', replace=[], children=True, newlines=True, normalize='NFC', **kwargs):
         super(CleanText, self).__init__(selector, **kwargs)
         self.symbols = symbols
         self.toreplace = replace
-        self.childs = childs
+        self.children = children
         self.newlines = newlines
+        self.normalize = normalize
 
     @debug()
     def filter(self, txt):
         if isinstance(txt, (tuple, list)):
-            txt = u' '.join([self.clean(item, childs=self.childs) for item in txt])
+            txt = u' '.join([self.clean(item, children=self.children) for item in txt])
 
-        txt = self.clean(txt, childs=self.childs, newlines=self.newlines)
+        txt = self.clean(txt, self.children, self.newlines, self.normalize)
         txt = self.remove(txt, self.symbols)
         txt = self.replace(txt, self.toreplace)
-        # lxml under Python 2 returns str instead of unicode if it is pure ASCII
+        # ensure it didn't become str by mistake
         return unicode(txt)
 
     @classmethod
-    def clean(cls, txt, childs=True, newlines=True):
+    def clean(cls, txt, children=True, newlines=True, normalize='NFC'):
         if not isinstance(txt, basestring):
-            if childs:
+            if children:
                 txt = [t.strip() for t in txt.itertext()]
             else:
-                txt = [txt.text.strip()]
+                txt = [t.strip() for t in txt.xpath('./text()')]
             txt = u' '.join(txt)  # 'foo   bar'
         if newlines:
             txt = re.compile(u'\s+', flags=re.UNICODE).sub(u' ', txt)  # 'foo bar'
         else:
             # normalize newlines and clean what is inside
             txt = '\n'.join([cls.clean(l) for l in txt.splitlines()])
-        return txt.strip()
+        txt = txt.strip()
+        # lxml under Python 2 returns str instead of unicode if it is pure ASCII
+        txt = unicode(txt)
+        # normalize to a standard Unicode form
+        if normalize:
+            txt = unicodedata.normalize(normalize, txt)
+        return txt
 
     @classmethod
     def remove(cls, txt, symbols):
@@ -364,6 +300,27 @@ class Lower(CleanText):
         return txt.lower()
 
 
+class Upper(CleanText):
+    @debug()
+    def filter(self, txt):
+        txt = super(Upper, self).filter(txt)
+        return txt.upper()
+
+
+class Capitalize(CleanText):
+    @debug()
+    def filter(self, txt):
+        txt = super(Capitalize, self).filter(txt)
+        return txt.title()
+
+
+class Currency(CleanText):
+    @debug()
+    def filter(self, txt):
+        txt = super(Currency, self).filter(txt)
+        return BaseCurrency.get_currency(txt)
+
+
 class CleanDecimal(CleanText):
     """
     Get a cleaned Decimal value from an element.
@@ -390,6 +347,9 @@ class CleanDecimal(CleanText):
 
     @debug()
     def filter(self, text):
+        if type(text) in (float, int, long):
+            text = str(text)
+
         if empty(text):
             return self.default_or_raise(ParseError('Unable to parse %r' % text))
 
@@ -409,6 +369,14 @@ class CleanDecimal(CleanText):
             return self.default_or_raise(e)
 
 
+class Slugify(Filter):
+    @debug()
+    def filter(self, label):
+        label = re.sub(r'[^A-Za-z0-9]', ' ', label.lower()).strip()
+        label = re.sub(r'\s+', '-', label)
+        return label
+
+
 class Type(Filter):
     """
     Get a cleaned value of any type from an element text.
@@ -418,14 +386,16 @@ class Type(Filter):
 
     >>> Type(CleanText('./td[1]'), type=int)  # doctest: +SKIP
 
+    >>> Type(type=int).filter(42)
+    42
     >>> Type(type=int).filter('42')
     42
     >>> Type(type=int, default='NaN').filter('')
     'NaN'
-    >>> Type(type=str, minlen=False, default='a').filter('')
-    ''
-    >>> Type(type=str, minlen=0, default='a').filter('')
-    'a'
+    >>> Type(type=list, minlen=False, default=list('ab')).filter('')
+    []
+    >>> Type(type=list, minlen=0, default=list('ab')).filter('')
+    ['a', 'b']
     """
 
     def __init__(self, selector=None, type=None, minlen=0, default=_NO_DEFAULT):
@@ -435,6 +405,8 @@ class Type(Filter):
 
     @debug()
     def filter(self, txt):
+        if isinstance(txt, self.type_func):
+            return txt
         if empty(txt):
             return self.default_or_raise(ParseError('Unable to parse %r' % txt))
         if self.minlen is not False and len(txt) <= self.minlen:
@@ -460,7 +432,9 @@ class Field(_Filter):
 
 # Based on nth from https://docs.python.org/2/library/itertools.html
 def nth(iterable, n, default=None):
-    "Returns the nth item or a default value, n can be negative"
+    "Returns the nth item or a default value, n can be negative, or '*' for all"
+    if n == '*':
+        return iterable
     if n < 0:
         iterable = reversed(list(iterable))
         n = abs(n) - 1
@@ -468,7 +442,9 @@ def nth(iterable, n, default=None):
 
 
 def ordinal(n):
-    "To have some readable debug information: 0 => 1st, 1 => 2nd..."
+    "To have some readable debug information: '*' => all, 0 => 1st, 1 => 2nd..."
+    if n == '*':
+        return 'all'
     i = abs(n)
     n = n - 1 if n < 0 else n + 1
     return str(n) + ('th' if i > 2 else ['st', 'nd', 'rd'][i])
@@ -487,6 +463,8 @@ class Regexp(Filter):
     u'08'
     >>> (Regexp(CleanText('//body'), r'(\d+)', nth=-1))(doc)
     u'1988'
+    >>> (Regexp(CleanText('//body'), r'(\d+)', template='[\\1]', nth='*'))(doc)
+    [u'[13]', u'[08]', u'[1988]']
     """
 
     def __init__(self, selector=None, pattern=None, template=None, nth=0, flags=0, default=_NO_DEFAULT):
@@ -497,21 +475,26 @@ class Regexp(Filter):
         self.template = template
         self.nth = nth
 
+    def expand(self, m):
+        if self.template is None:
+            return next(g for g in m.groups() if g is not None)
+        return self.template(m) if callable(self.template) else m.expand(self.template)
+
     @debug()
     def filter(self, txt):
         if isinstance(txt, (tuple, list)):
             txt = u' '.join([t.strip() for t in txt.itertext()])
 
-        mobj = self._regex.search(txt) if self.nth == 0 else \
-               nth(self._regex.finditer(txt), self.nth)
-        if not mobj:
-            msg = 'Unable to match %s %s in %r' % (ordinal(self.nth), self.pattern, txt)
+        m = self._regex.search(txt) if self.nth == 0 else \
+            nth(self._regex.finditer(txt), self.nth)
+        if not m:
+            msg = 'Unable to find %s %s in %r' % (ordinal(self.nth), self.pattern, txt)
             return self.default_or_raise(RegexpError(msg))
 
-        if self.template is None:
-            return next(g for g in mobj.groups() if g is not None)
-        else:
-            return self.template(mobj) if callable(self.template) else mobj.expand(self.template)
+        if isinstance(m, Iterator):
+            return list(map(self.expand, m))
+
+        return self.expand(m)
 
 
 class Map(Filter):
@@ -529,10 +512,13 @@ class Map(Filter):
 
 
 class DateTime(Filter):
-    def __init__(self, selector=None, default=_NO_DEFAULT, dayfirst=False, translations=None):
+    def __init__(self, selector=None, default=_NO_DEFAULT, dayfirst=False, translations=None,
+                 parse_func=parse_date, fuzzy=False):
         super(DateTime, self).__init__(selector, default=default)
         self.dayfirst = dayfirst
         self.translations = translations
+        self.parse_func = parse_func
+        self.fuzzy = fuzzy
 
     @debug()
     def filter(self, txt):
@@ -542,14 +528,16 @@ class DateTime(Filter):
             if self.translations:
                 for search, repl in self.translations:
                     txt = search.sub(repl, txt)
-            return parse_date(txt, dayfirst=self.dayfirst)
-        except ValueError as e:
+            return self.parse_func(txt, dayfirst=self.dayfirst, fuzzy=self.fuzzy)
+        except (ValueError, TypeError) as e:
             return self.default_or_raise(ParseError('Unable to parse %r: %s' % (txt, e)))
 
 
 class Date(DateTime):
-    def __init__(self, selector=None, default=_NO_DEFAULT, dayfirst=False, translations=None):
-        super(Date, self).__init__(selector, default=default, dayfirst=dayfirst, translations=translations)
+    def __init__(self, selector=None, default=_NO_DEFAULT, dayfirst=False, translations=None,
+                 parse_func=parse_date, fuzzy=False):
+        super(Date, self).__init__(selector, default=default, dayfirst=dayfirst, translations=translations,
+                                   parse_func=parse_func, fuzzy=fuzzy)
 
     @debug()
     def filter(self, txt):
@@ -567,11 +555,11 @@ class DateGuesser(Filter):
         self.kwargs = kwargs
 
     def __call__(self, item):
-        values = self.select(self.selector, item, obj=self._obj, key=self._key)
+        values = self.select(self.selector, item)
         date_guesser = self.date_guesser
         # In case Env() is used to kive date_guesser.
         if isinstance(date_guesser, _Filter):
-            date_guesser = self.select(date_guesser, item, obj=self._obj, key=self._key)
+            date_guesser = self.select(date_guesser, item)
 
         if isinstance(values, basestring):
             values = re.split('[/-]', values)
@@ -584,7 +572,7 @@ class DateGuesser(Filter):
 
 class Time(Filter):
     klass = datetime.time
-    _regexp = re.compile(r'(?P<hh>\d+):?(?P<mm>\d+)(:(?P<ss>\d+))?')
+    _regexp = re.compile(r'(?P<hh>\d+)[:h]?(?P<mm>\d+)([:m](?P<ss>\d+))?')
     kwargs = {'hour': 'hh', 'minute': 'mm', 'second': 'ss'}
 
     def __init__(self, selector=None, default=_NO_DEFAULT):
@@ -595,7 +583,7 @@ class Time(Filter):
         m = self._regexp.search(txt)
         if m:
             kwargs = {}
-            for key, index in self.kwargs.iteritems():
+            for key, index in self.kwargs.items():
                 kwargs[key] = int(m.groupdict()[index] or 0)
             return self.klass(**kwargs)
 
@@ -604,7 +592,7 @@ class Time(Filter):
 
 class Duration(Time):
     klass = datetime.timedelta
-    regexp = re.compile(r'((?P<hh>\d+)[:;])?(?P<mm>\d+)[;:](?P<ss>\d+)')
+    _regexp = re.compile(r'((?P<hh>\d+)[:;])?(?P<mm>\d+)[;:](?P<ss>\d+)')
     kwargs = {'hours': 'hh', 'minutes': 'mm', 'seconds': 'ss'}
 
 
@@ -614,7 +602,7 @@ class MultiFilter(Filter):
         super(MultiFilter, self).__init__(args, default)
 
     def __call__(self, item):
-        values = [self.select(selector, item, obj=self._obj, key=self._key) for selector in self.selector]
+        values = [self.select(selector, item) for selector in self.selector]
         return self.filter(tuple(values))
 
     def filter(self, values):
@@ -644,7 +632,7 @@ class BrowserURL(MultiFilter):
     def __init__(self, url_name, **kwargs):
         super(BrowserURL, self).__init__(*kwargs.values())
         self.url_name = url_name
-        self.keys = kwargs.keys()
+        self.keys = list(kwargs.keys())
 
     def __call__(self, item):
         values = super(BrowserURL, self).__call__(item)
@@ -658,21 +646,85 @@ class BrowserURL(MultiFilter):
 
 
 class Join(Filter):
-    def __init__(self, pattern, selector=None, textCleaner=CleanText):
+    def __init__(self, pattern, selector=None, textCleaner=CleanText, newline=False, addBefore='', addAfter=''):
         super(Join, self).__init__(selector)
         self.pattern = pattern
         self.textCleaner = textCleaner
+        self.newline = newline
+        self.addBefore = addBefore
+        self.addAfter = addAfter
 
     @debug()
     def filter(self, el):
-        res = u''
-        for li in el:
-            res += self.pattern % self.textCleaner.clean(li)
+        items = [self.textCleaner.clean(e) for e in el]
+        items = [item for item in items if item]
 
-        return res
+        if self.newline:
+            items = ['%s\r\n' % item for item in items]
+
+        result = self.pattern.join(items)
+
+        if self.addBefore:
+            result = '%s%s' % (self.addBefore, result)
+
+        if self.addAfter:
+            result = '%s%s' % (result, self.addAfter)
+
+        return result
 
 
-def test():
+class Eval(MultiFilter):
+    """
+    Evaluate a function with given 'deferred' arguments.
+
+    >>> F = Field; Eval(lambda a, b, c: a * b + c, F('foo'), F('bar'), F('baz')) # doctest: +SKIP
+    >>> Eval(lambda x, y: x * y + 1).filter([3, 7])
+    22
+    """
+    def __init__(self, func, *args):
+        super(Eval, self).__init__(*args)
+        self.func = func
+
+    @debug()
+    def filter(self, values):
+        return self.func(*values)
+
+
+class QueryValue(Filter):
+    """
+    Extract the value of a parameter from an URL with a query string.
+
+    >>> from lxml.html import etree
+    >>> from .html import Link
+    >>> f = QueryValue(Link('//a'), 'id')
+    >>> f(etree.fromstring('<html><body><a href="http://example.org/view?id=1234"></a></body></html>'))
+    u'1234'
+    """
+    def __init__(self, selector, key, default=_NO_DEFAULT):
+        super(QueryValue, self).__init__(selector, default=default)
+        self.querykey = key
+
+    @debug()
+    def filter(self, url):
+        qs = parse_qs(urlparse(url).query)
+        if not qs.get(self.querykey):
+            return self.default_or_raise(ParseError('Key %s not found' % self.querykey))
+        if len(qs[self.querykey]) > 1:
+            raise ParseError('More than one value for key %s' % self.querykey)
+        return qs[self.querykey][0]
+
+
+def test_CleanText():
     # This test works poorly under a doctest, or would be hard to read
     assert CleanText().filter(u' coucou  \n\théhé') == u'coucou héhé'
     assert CleanText().filter('coucou\xa0coucou') == CleanText().filter(u'coucou\xa0coucou') == u'coucou coucou'
+
+    # Unicode normalization
+    assert CleanText().filter(u'Éçã') == u'Éçã'
+    assert CleanText(normalize='NFKC').filter(u'…') == u'...'
+    assert CleanText().filter(u'…') == u'…'
+    # Diacritical mark (dakuten)
+    assert CleanText().filter(u'\u3053\u3099') == u'\u3054'
+    assert CleanText(normalize='NFD').filter(u'\u3053\u3099') == u'\u3053\u3099'
+    assert CleanText(normalize='NFD').filter(u'\u3054') == u'\u3053\u3099'
+    assert CleanText(normalize=False).filter(u'\u3053\u3099') == u'\u3053\u3099'
