@@ -33,32 +33,34 @@ try:
 except ImportError:
     raise ImportError('Please install python-mechanize')
 
-import os
-import re
-from threading import RLock
-import ssl
-import httplib
-import socket
+import base64
 import hashlib
+import httplib
+import logging
+import mimetypes
+import os
+import pickle
+import re
+import socket
+import ssl
 import time
 import urllib
 import urllib2
-import mimetypes
-import logging
+import warnings
+import zlib
 from contextlib import closing
 from gzip import GzipFile
-import warnings
+from threading import RLock
 
 from weboob.exceptions import BrowserUnavailable, BrowserIncorrectPassword, BrowserPasswordExpired, BrowserForbidden, BrowserBanned, BrowserHTTPNotFound, BrowserHTTPError, FormFieldConversionWarning, BrowserSSLError
+from weboob.tools.compat import basestring, unicode
 from weboob.tools.decorators import retry
 from weboob.tools.log import getLogger
-from weboob.deprecated.mech import ClientForm
-ControlNotFoundError = ClientForm.ControlNotFoundError
 from weboob.deprecated.browser.parsers import get_parser
 
 __all__ = ['BrowserIncorrectPassword', 'BrowserForbidden', 'BrowserBanned', 'BrowserUnavailable', 'BrowserRetry',
            'BrowserPasswordExpired', 'BrowserHTTPNotFound', 'BrowserHTTPError', 'BrokenPageError', 'Page',
-           'StandardBrowser', 'Browser']
+           'StandardBrowser', 'Browser', 'StateBrowser']
 
 
 class BrowserRetry(Exception):
@@ -149,7 +151,7 @@ class StandardBrowser(mechanize.Browser):
 
     ENCODING = 'utf-8'
     USER_AGENTS = {
-        'desktop_firefox': 'Mozilla/5.0 (X11; Linux x86_64; rv:31.0) Gecko/20100101 Firefox/31.0',
+        'desktop_firefox': 'Mozilla/5.0 (X11; Linux x86_64; rv:45.0) Gecko/20100101 Firefox/45.0',
         'android': 'Mozilla/5.0 (Linux; U; Android 2.1; en-us; Nexus One Build/ERD62) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17',
         'microb': 'Mozilla/5.0 (X11; U; Linux armv7l; fr-FR; rv:1.9.2.3pre) Gecko/20100723 Firefox/3.5 Maemo Browser 1.7.4.8 RX-51 N900',
         'wget': 'Wget/1.11.4',
@@ -172,6 +174,9 @@ class StandardBrowser(mechanize.Browser):
     def __init__(self, firefox_cookies=None, parser=None, history=NoHistory(), proxy=None, logger=None, factory=None, responses_dirname=None):
         mechanize.Browser.__init__(self, history=history, factory=factory)
         self.logger = getLogger('browser', logger)
+        self.responses_dirname = responses_dirname
+        self.save_responses = responses_dirname is not None
+        self.responses_count = 0
 
         self.addheaders = [
                 ['User-agent', self.USER_AGENT]
@@ -213,9 +218,6 @@ class StandardBrowser(mechanize.Browser):
             mech_logger = logging.getLogger("mechanize")
             mech_logger.setLevel(logging.INFO)
 
-        self.responses_dirname = responses_dirname
-        self.responses_count = 0
-
     def __enter__(self):
         self.lock.acquire()
 
@@ -270,7 +272,7 @@ class StandardBrowser(mechanize.Browser):
         result = self.openurl(url, *args, **kwargs)
 
         if result:
-            if self.logger.settings['save_responses']:
+            if self.save_responses:
                 self.save_response(result)
             return result.read()
         else:
@@ -394,11 +396,11 @@ class StandardBrowser(mechanize.Browser):
                     else:
                         value = self.str(args[label])
                 self[field] = value
-        except ControlNotFoundError:
+        except mechanize.ControlNotFoundError:
             return
 
     def lowsslcheck(self, domain, hsh):
-        if self.INSECURE or (self.logger is not None and self.logger.settings['ssl_insecure']):
+        if self.INSECURE or (self.logger is not None and self.logger.settings['ssl_insecure']) or self.proxy is not None:
             return
         certhash = self._certhash(domain)
         if self.logger:
@@ -628,12 +630,7 @@ class Browser(StandardBrowser):
                 response.set_data(data)
         mechanize.Browser._set_response(self, response, *args, **kwargs)
 
-    def _change_location(self, result, no_login=False):
-        """
-        This function is called when we have moved to a page, to load a Page
-        object.
-        """
-
+    def get_page(self, result):
         # Find page from url
         pageCls = None
         parser = None
@@ -662,19 +659,29 @@ class Browser(StandardBrowser):
 
         # Not found
         if not pageCls:
-            self.page = None
-            self.logger.warning('There isn\'t any page corresponding to URL %s' % result.geturl())
+            self.logger.warning('There isn\'t any page corresponding to URL %s', result.geturl())
             self.save_response(result, warning=True)
             return
 
-        self.logger.debug('[user_id=%s] Went on %s' % (self.username, result.geturl()))
+        self.logger.debug('[user_id=%s] Went on %s', self.username, result.geturl())
         self.last_update = time.time()
 
-        if self.logger.settings['save_responses']:
+        if self.save_responses:
             self.save_response(result)
 
         document = self.get_document(result, parser, encoding=pageCls.ENCODING)
-        self.page = pageCls(self, document, result.geturl(), groups=page_groups, group_dict=page_group_dict, logger=self.logger)
+        return pageCls(self, document, result.geturl(), groups=page_groups, group_dict=page_group_dict, logger=self.logger)
+
+    def _change_location(self, result, no_login=False):
+        """
+        This function is called when we have moved to a page, to load a Page
+        object.
+        """
+
+        self.page = self.get_page(result)
+
+        if not self.page:
+            return
 
         if not no_login and self.password is not None and not self.is_logged():
             self.logger.debug('!! Relogin !!')
@@ -687,6 +694,48 @@ class Browser(StandardBrowser):
             self._cookie.save()
 
 
+class StateBrowser(Browser):
+    """
+    This browser aims to store state (cookies, location and attributes).
+    """
+
+    __states__ = []
+    """
+    Saved state variables.
+    """
+
+    def __init__(self, *args, **kwargs):
+        kwargs['get_home'] = False
+        Browser.__init__(self, *args, **kwargs)
+
+    def load_state(self, state):
+        if 'cookies' in state:
+            try:
+                cookies = pickle.loads(zlib.decompress(base64.b64decode(state['cookies'])))
+            except (TypeError, zlib.error, EOFError, ValueError):
+                self.logger.error('Unable to reload cookies from storage')
+            else:
+                self.set_cookiejar(cookies)
+                self.logger.info('Reloaded cookies from storage')
+        for attrname in self.__states__:
+            if attrname in state:
+                setattr(self, attrname, state[attrname])
+        if 'url' in state:
+            self.location(state['url'], nologin=True)
+
+
+    def dump_state(self):
+        state = {}
+        cookiejar = self._ua_handlers["_cookies"].cookiejar
+        state['cookies'] = base64.b64encode(zlib.compress(pickle.dumps(cookiejar, -1)))
+        for attrname in self.__states__:
+            state[attrname] = getattr(self, attrname)
+        if self.page:
+            state['url'] = self.page.url
+        self.logger.info('Stored cookies into storage')
+        return state
+
+
 def mywrap_socket(sock, *args, **kwargs):
     kwargs['do_handshake_on_connect'] = False
     sock = ssl.wrap_socketold(sock, *args, **kwargs)
@@ -704,6 +753,11 @@ ssl.wrap_socketold = ssl.wrap_socket
 ssl.wrap_socket = mywrap_socket
 
 
+# XXX MONKEY PATCHING TO DISABLE SSL CERTIFICATE VERIFICATION
+if hasattr(ssl, '_create_unverified_context'):
+    ssl._create_default_https_context = ssl._create_unverified_context
+
+
 class DNSTimeoutException(Exception):
     pass
 
@@ -730,7 +784,7 @@ socket.getaddrinfo = my_getaddrinfo
 
 class HTTPSConnection2(httplib.HTTPSConnection):
     _HOSTS = {}
-    _PROTOCOLS = [ssl.PROTOCOL_SSLv23]
+    _PROTOCOLS = [getattr(ssl, pn) for pn in ['PROTOCOL_TLSv1_2', 'PROTOCOL_TLSv1_1', 'PROTOCOL_TLSv1', 'PROTOCOL_SSLv23'] if hasattr(ssl, pn)]
 
     def _my_create_connection(self):
         sock = socket.create_connection((self.host, self.port), self.timeout)
