@@ -30,14 +30,18 @@ class BrowserBanned(BrowserIncorrectPassword):
     pass
 
 
-class BrowserPasswordExpired(BrowserIncorrectPassword):
-    pass
-
-
 class BrowserUnavailable(Exception):
     pass
 
 
+class BrowserQuestion(BrowserIncorrectPassword):
+    """
+    When raised by a browser,
+    """
+    def __init__(self, *fields):
+        self.fields = fields
+
+
 class BrowserHTTPNotFound(BrowserUnavailable):
     pass
 
@@ -46,6 +50,10 @@ class BrowserHTTPError(BrowserUnavailable):
     pass
 
 
+class BrowserHTTPSDowngrade(BrowserUnavailable):
+    pass
+
+
 class BrowserSSLError(BrowserUnavailable):
     pass
 
@@ -58,3 +66,25 @@ class FormFieldConversionWarning(UserWarning):
     """
     A value has been set to a form's field and has been implicitly converted.
     """
+
+
+class NoAccountsException(Exception):
+    pass
+
+
+class ModuleInstallError(Exception):
+    pass
+
+
+class ModuleLoadError(Exception):
+    def __init__(self, module_name, msg):
+        super(ModuleLoadError, self).__init__(msg)
+        self.module = module_name
+
+
+class ActionNeeded(Exception):
+    pass
+
+
+class BrowserPasswordExpired(ActionNeeded):
+    pass
